/*!
 @its2easy/animate-sprite 2.2.1
 https://github.com/its2easy/animate-sprite
         
 Copyright (c) 2020-present Dmitry Kovalev,
 Released under the MIT license
*/
function _defineProperty(obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }

  return obj;
}

function _classPrivateFieldGet(receiver, privateMap) {
  var descriptor = _classExtractFieldDescriptor(receiver, privateMap, "get");

  return _classApplyDescriptorGet(receiver, descriptor);
}

function _classPrivateFieldSet(receiver, privateMap, value) {
  var descriptor = _classExtractFieldDescriptor(receiver, privateMap, "set");

  _classApplyDescriptorSet(receiver, descriptor, value);

  return value;
}

function _classExtractFieldDescriptor(receiver, privateMap, action) {
  if (!privateMap.has(receiver)) {
    throw new TypeError("attempted to " + action + " private field on non-instance");
  }

  return privateMap.get(receiver);
}

function _classApplyDescriptorGet(receiver, descriptor) {
  if (descriptor.get) {
    return descriptor.get.call(receiver);
  }

  return descriptor.value;
}

function _classApplyDescriptorSet(receiver, descriptor, value) {
  if (descriptor.set) {
    descriptor.set.call(receiver, value);
  } else {
    if (!descriptor.writable) {
      throw new TypeError("attempted to set read only private field");
    }

    descriptor.value = value;
  }
}

function _classPrivateMethodGet(receiver, privateSet, fn) {
  if (!privateSet.has(receiver)) {
    throw new TypeError("attempted to get private field on non-instance");
  }

  return fn;
}

function _checkPrivateRedeclaration(obj, privateCollection) {
  if (privateCollection.has(obj)) {
    throw new TypeError("Cannot initialize the same private elements twice on an object");
  }
}

function _classPrivateFieldInitSpec(obj, privateMap, value) {
  _checkPrivateRedeclaration(obj, privateMap);

  privateMap.set(obj, value);
}

function _classPrivateMethodInitSpec(obj, privateSet) {
  _checkPrivateRedeclaration(obj, privateSet);

  privateSet.add(obj);
}

/**
 * Returns validated frame number between the first and the last frame
 * @param {Number} frameNumber
 * @param {Number} totalFrames
 * @returns {Number}
 */
function normalizeFrameNumber(frameNumber, totalFrames) {
  frameNumber = Math.floor(frameNumber);

  if (frameNumber <= 0) {
    return 1;
  } else if (frameNumber > totalFrames) {
    return totalFrames;
  }

  return frameNumber;
}
function capitalizeFirstLetter(string) {
  return string.charAt(0).toUpperCase() + string.slice(1);
}
function isNumeric(str) {
  return !isNaN(str) && // use type coercion to parse the _entirety_ of the string (`parseFloat` alone does not do this)...
  !isNaN(parseFloat(str)); // ...and ensure strings of whitespace fail
}

function validateParameters(node, options) {
  if (!(node instanceof HTMLElement || node instanceof Element || node instanceof HTMLDocument)) {
    throw new TypeError('Node is required');
  }
}
function validateSettings(options) {
  ["width", "height", "frames"].forEach(el => {
    options[el] = parseInt(options[el], 10);
    if (!isNumeric(options[el])) throw new TypeError(`options.${el} is required and must be a number`);
  });
  ["cols", "frameTime", "fps", "duration", "pageScrollTimerDelay"].forEach(el => {
    if (options[el]) options[el] = parseInt(options[el], 10);
  });
  if ('dragModifier' in options) options.dragModifier = Math.abs(+options.dragModifier);
}
function getSettings(node, options) {
  let inlineSettings = fillInlineSettings(node, getSettingsKeys());
  return { ...defaultSettings,
    ...options,
    ...inlineSettings
  };
}
const defaultSettings = {
  width: undefined,
  height: undefined,
  frames: undefined,
  cols: false,
  loop: false,
  autoplay: false,
  frameTime: false,
  duration: false,
  fps: 24,
  reverse: false,
  draggable: false,
  inversion: false,
  dragModifier: 1,
  touchScrollMode: "pageScrollTimer",
  pageScrollTimerDelay: 1500,
  onAnimationEnd: undefined,
  onAfterFrameChange: undefined
};
function getSettingsKeys() {
  return Object.keys(defaultSettings);
}
/**
 * Returns object with data-* settings
 *
 * @param {HTMLElement} node
 * @param {Array<string>} list
 * @returns {Object}
 */

function fillInlineSettings(node, list) {
  let result = {};
  const prefix = "sprite";
  list.forEach(value => {
    const datasetFormattedValue = prefix + capitalizeFirstLetter(value);
    let inlineValue = node.dataset[datasetFormattedValue];

    if (typeof inlineValue !== 'undefined') {
      if (inlineValue === "true" || inlineValue === '') inlineValue = true;else if (inlineValue === "false") inlineValue = false;else if (isNumeric(inlineValue)) inlineValue = parseFloat(inlineValue);
      result[value] = inlineValue;
    }
  });
  return result;
}

var _swipeHandler = /*#__PURE__*/new WeakSet();

var _swipeStart = /*#__PURE__*/new WeakSet();

var _swipeMove = /*#__PURE__*/new WeakSet();

var _swipeEnd = /*#__PURE__*/new WeakSet();

var _swipeDirection = /*#__PURE__*/new WeakSet();

var _maybeDisableScroll = /*#__PURE__*/new WeakSet();

var _toggleDocumentEventHandlers = /*#__PURE__*/new WeakSet();

class DragInput {
  constructor(_ref) {
    let {
      data,
      settings,
      changeFrame,
      getNextFrame
    } = _ref;

    _classPrivateMethodInitSpec(this, _toggleDocumentEventHandlers);

    _classPrivateMethodInitSpec(this, _maybeDisableScroll);

    _classPrivateMethodInitSpec(this, _swipeDirection);

    _classPrivateMethodInitSpec(this, _swipeEnd);

    _classPrivateMethodInitSpec(this, _swipeMove);

    _classPrivateMethodInitSpec(this, _swipeStart);

    _classPrivateMethodInitSpec(this, _swipeHandler);

    _defineProperty(this, "data", void 0);

    _defineProperty(this, "settings", void 0);

    _defineProperty(this, "changeFrame", void 0);

    _defineProperty(this, "getNextFrame", void 0);

    _defineProperty(this, "boundSwipeHandler", void 0);

    _defineProperty(this, "curX", void 0);

    _defineProperty(this, "curY", void 0);

    _defineProperty(this, "prevX", void 0);

    _defineProperty(this, "prevY", void 0);

    _defineProperty(this, "isSwiping", false);

    _defineProperty(this, "threshold", void 0);

    _defineProperty(this, "pixelsCorrection", 0);

    _defineProperty(this, "lastInteractionTime", void 0);

    this.data = data;
    this.settings = settings;
    this.changeFrame = changeFrame;
    this.getNextFrame = getNextFrame;
    this.boundSwipeHandler = _classPrivateMethodGet(this, _swipeHandler, _swipeHandler2).bind(this);
    this.updateThreshold();
  }

  enableDrag() {
    DragInput.SWIPE_EVENTS.forEach(value => {
      this.data.element.addEventListener(value, this.boundSwipeHandler);
    });
  }

  disableDrag() {
    DragInput.SWIPE_EVENTS.forEach(value => {
      this.data.element.removeEventListener(value, this.boundSwipeHandler);
    });

    _classPrivateMethodGet(this, _toggleDocumentEventHandlers, _toggleDocumentEventHandlers2).call(this, false);

    this.data.element.style.cursor = null;
  }
  /**
   * Update one frame threshold in pixels, based on DOM element width, should be called after nodeWidth change
   * @param newValue
   */


  updateThreshold() {
    let newValue = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : null;

    if (newValue) {
      this.threshold = newValue;
    } else {
      this.threshold = this.data.nodeWidth / this.settings.frames;
    }
  }

}

function _swipeHandler2(event) {
  let touches;
  if (event.touches !== undefined && event.touches.length) touches = event.touches;
  this.curX = touches ? touches[0].pageX : event.clientX;
  this.curY = touches ? touches[0].pageY : event.clientY;

  switch (event.type) {
    case 'mousedown':
    case 'touchstart':
      if (event.type === 'touchstart' && event.cancelable) _classPrivateMethodGet(this, _maybeDisableScroll, _maybeDisableScroll2).call(this, event);

      _classPrivateMethodGet(this, _toggleDocumentEventHandlers, _toggleDocumentEventHandlers2).call(this, true);

      _classPrivateMethodGet(this, _swipeStart, _swipeStart2).call(this);

      break;

    case 'mousemove':
    case 'touchmove':
      if (this.isSwiping) _classPrivateMethodGet(this, _swipeMove, _swipeMove2).call(this);
      break;

    case 'mouseup':
    case 'touchend':
    case 'touchcancel':
      //if ( (event.type === 'touchend' || event.type === 'touchcancel') && event.cancelable) event.preventDefault();
      _classPrivateMethodGet(this, _toggleDocumentEventHandlers, _toggleDocumentEventHandlers2).call(this, false);

      _classPrivateMethodGet(this, _swipeEnd, _swipeEnd2).call(this);

      break;
  }
}

function _swipeStart2() {
  this.data.pluginApi.stop();
  this.isSwiping = true;
  this.data.element.style.cursor = 'grabbing';
  this.prevX = this.curX;
  this.prevY = this.curY;
  this.data.element.dispatchEvent(new CustomEvent('sprite:drag-start', {
    detail: {
      frame: this.data.currentFrame
    }
  }));
}

function _swipeMove2() {
  const direction = _classPrivateMethodGet(this, _swipeDirection, _swipeDirection2).call(this);

  const swipeLength = Math.round(Math.abs(this.curX - this.prevX) * this.settings.dragModifier) + this.pixelsCorrection;
  if (swipeLength <= this.threshold) return; // Ignore if pixels are less than 1 frame

  if (direction !== 'left' && direction !== 'right') return; // Ignore vertical directions

  this.prevX = this.curX;
  this.prevY = this.curY;
  const progress = swipeLength / this.data.nodeWidth;
  let deltaFrames = Math.floor(progress * this.settings.frames);
  deltaFrames = deltaFrames % this.settings.frames; // Add pixels to the next swipeMove if frames equivalent of swipe is not an integer number,
  // e.g one frame is 10px, swipeLength is 13px, we change 1 frame and add 3px to the next swipe,
  // so fullwidth swipe is always rotate sprite for 1 turn / all frames (with 'dragModifier' =  1)

  this.pixelsCorrection = swipeLength - this.threshold * deltaFrames;
  let isReverse = direction === 'left'; // left means backward (reverse: true)

  if (this.settings.inversion) isReverse = !isReverse; // invert direction

  this.changeFrame(this.getNextFrame(deltaFrames, isReverse));
  this.data.element.dispatchEvent(new CustomEvent('sprite:drag-change', {
    detail: {
      frame: this.data.currentFrame
    }
  }));
}

function _swipeEnd2() {
  this.curX = this.curY = this.prevX = this.prevY = null;
  this.isSwiping = false;
  this.data.element.style.cursor = null;
  this.lastInteractionTime = new Date().getTime();
  this.data.element.dispatchEvent(new CustomEvent('sprite:drag-end', {
    detail: {
      frame: this.data.currentFrame
    }
  }));
}

function _swipeDirection2() {
  let xDist, yDist, r, swipeAngle;
  xDist = this.prevX - this.curX;
  yDist = this.prevY - this.curY;
  r = Math.atan2(yDist, xDist);
  swipeAngle = Math.round(r * 180 / Math.PI);
  if (swipeAngle < 0) swipeAngle = 360 - Math.abs(swipeAngle);
  if (swipeAngle >= 0 && swipeAngle <= 60 || swipeAngle <= 360 && swipeAngle >= 300) return 'left';else if (swipeAngle >= 120 && swipeAngle <= 240) return 'right';else if (swipeAngle >= 241 && swipeAngle <= 299) return 'bottom';else return 'up';
}

function _maybeDisableScroll2(event) {
  // always prevent
  if (this.settings.touchScrollMode === "preventPageScroll") event.preventDefault(); // check timer

  if (this.settings.touchScrollMode === "pageScrollTimer") {
    let now = new Date().getTime(); // less time than delay => prevent page scroll

    if (this.lastInteractionTime && now - this.lastInteractionTime < this.settings.pageScrollTimerDelay) {
      event.preventDefault();
    } else {
      // more time than delay or first interaction => clear timer
      this.lastInteractionTime = null;
    }
  } // if touchScrollMode="allowPageScroll" => don't prevent scroll, so no action is needed

}

function _toggleDocumentEventHandlers2() {
  let add = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : true;

  if (add) {
    document.addEventListener('mouseup', this.boundSwipeHandler);
    document.addEventListener('mousemove', this.boundSwipeHandler);
  } else {
    document.removeEventListener('mouseup', this.boundSwipeHandler);
    document.removeEventListener('mousemove', this.boundSwipeHandler);
  }
}

_defineProperty(DragInput, "SWIPE_EVENTS", ['mousedown', 'mousemove', 'mouseup', 'touchstart', 'touchmove', 'touchend', 'touchcancel']);

var _animate = /*#__PURE__*/new WeakSet();

var _calculateDuration = /*#__PURE__*/new WeakSet();

class Animation {
  // frames from playTo() and playFrames()
  // time from RAF
  // time of the full animation sequence
  // save decimal part if deltaFrames is not round, to prevent rounding errors
  // >35% means that there was a long task in callstack
  constructor(_ref) {
    let {
      settings,
      data,
      changeFrame
    } = _ref;

    _classPrivateMethodInitSpec(this, _calculateDuration);

    _classPrivateMethodInitSpec(this, _animate);

    _defineProperty(this, "settings", void 0);

    _defineProperty(this, "data", void 0);

    _defineProperty(this, "changeFrame", void 0);

    _defineProperty(this, "isAnimating", false);

    _defineProperty(this, "framesLeftToPlay", undefined);

    _defineProperty(this, "boundAnimate", void 0);

    _defineProperty(this, "lastUpdate", void 0);

    _defineProperty(this, "duration", void 0);

    _defineProperty(this, "stopRequested", false);

    _defineProperty(this, "framesQueue", 0);

    _defineProperty(this, "progressThreshold", 0.35);

    this.settings = settings;
    this.data = data;
    this.changeFrame = changeFrame;
    this.boundAnimate = _classPrivateMethodGet(this, _animate, _animate2).bind(this);
    this.updateDuration();
  }

  play() {
    this.isAnimating = true;
    this.stopRequested = false; // fix for the case when stopRequested was set inside getNextFrame that was called outside #animate

    this.lastUpdate = null; // first 'lastUpdate' should be always set in the first raf of the current animation

    requestAnimationFrame(this.boundAnimate);
  }

  stop() {
    if (this.isAnimating) {
      this.data.element.dispatchEvent(new Event('sprite:animation-end'));
      if (this.settings.onAnimationEnd) this.settings.onAnimationEnd(this.data.pluginApi);
    }

    this.isAnimating = false;
    this.framesLeftToPlay = undefined;
  }

  getNextFrame(deltaFrames) {
    let reverse = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : undefined;
    deltaFrames = Math.floor(deltaFrames); //just to be safe
    // Handle reverse

    if (reverse === undefined) reverse = this.settings.reverse;
    let newFrameNumber = reverse ? this.data.currentFrame - deltaFrames : this.data.currentFrame + deltaFrames; // Handle loop

    if (this.settings.loop) {
      // loop and outside of the frames
      if (newFrameNumber <= 0) {
        // for example newFrame = -2, total = 50, newFrame = 50 - abs(-2) = 48
        newFrameNumber = this.settings.frames - Math.abs(newFrameNumber);
      } else if (newFrameNumber > this.settings.frames) {
        // for example newFrame = 53, total 50, newFrame = newFrame - totalFrames = 53 - 50 = 3
        newFrameNumber = newFrameNumber - this.settings.frames;
      }
    } else {
      // no loop and outside of the frames
      if (newFrameNumber <= 0) {
        newFrameNumber = 1;
        this.stopRequested = true;
      } else if (newFrameNumber > this.settings.frames) {
        newFrameNumber = this.settings.frames;
        this.stopRequested = true;
      }
    }

    return newFrameNumber;
  } // RAF callback
  // (chrome) 'timestamp' is timestamp from the moment the RAF callback was queued
  // (firefox) 'timestamp' is timestamp from the moment the RAF callback was called
  // the difference is equal to the time that the main thread was executing after raf callback was queued


  updateDuration() {
    this.duration = _classPrivateMethodGet(this, _calculateDuration, _calculateDuration2).call(this);
  }

}

function _animate2(timestamp) {
  if (!this.isAnimating) return; //stop() was called before this RAF
  // lastUpdate is setting here because the time between play() and #animate() is unpredictable, and
  // lastUpdate = performance.now() instead of timestamp because timestamp is unpredictable and depends on the browser.
  // Possible frame change in the first raf will always be skipped, because time <= performance.now

  if (!this.lastUpdate) this.lastUpdate = performance.now();
  let deltaFrames; // Check if there was a long task between this and the last frame, if so move 1 fixed frame and change lastUpdate to now
  // to prevent animation jump. (1,2,3,long task,75,76,77, ... => 1,2,3,long task,4,5,6,...)
  // In this case the duration will be longer

  let isLongTaskBeforeRaf = Math.abs(timestamp - performance.now()) / this.duration > this.progressThreshold; //chrome check

  let progress = (timestamp - this.lastUpdate) / this.duration; // ex. 0.01

  if (progress > this.progressThreshold) isLongTaskBeforeRaf = true; // firefox check

  if (isLongTaskBeforeRaf) deltaFrames = 1; // raf after long task, just move to the next frame
  else {
    // normal execution, calculate progress after the last frame change
    if (progress < 0) progress = 0; //it happens sometimes, when raf timestamp is from the past for some reason

    deltaFrames = progress * this.settings.frames; // for example, 0.45 or 1.25
    // e.g. progress is 0.8 frames, queue is 0.25 frames, so now deltaFrames is 1.05 frames and we need to update canvas,
    // without this raf intervals will cause cumulative rounding errors, and actual fps will decrease

    deltaFrames = deltaFrames + this.framesQueue;
  } // calculate next frame only when we want to render
  // if the getNextFrame check was outside, getNextFrame would be called at screen fps rate, not animation fps
  // if screen fps 144 and animation fps 30, getNextFrame is calling now 30/s instead of 144/s.
  // After the last frame, raf is repeating until the next frame calculation,
  // between the last frame drawing and new frame time, reverse or loop could be changed, and animation won't stop

  if (deltaFrames >= 1) {
    // Calculate only if we need to update 1 frame or more
    const newLastUpdate = isLongTaskBeforeRaf ? performance.now() : timestamp;
    this.framesQueue = deltaFrames % 1; // save decimal part for the next RAFs

    deltaFrames = Math.floor(deltaFrames) % this.settings.frames;
    if (deltaFrames > this.framesLeftToPlay) deltaFrames = this.framesLeftToPlay; // case when  animation fps > device fps

    const newFrame = this.getNextFrame(deltaFrames);

    if (this.stopRequested) {
      // animation ended from check in getNextFrame()
      this.data.pluginApi.stop();
      this.stopRequested = false;
      if (this.data.pluginApi.getCurrentFrame() !== newFrame) this.changeFrame(newFrame); //last frame fix if fps > device fps
    } else {
      // animation is on
      this.lastUpdate = newLastUpdate;
      this.changeFrame(newFrame);

      if (typeof this.framesLeftToPlay !== 'undefined') {
        this.framesLeftToPlay = this.framesLeftToPlay - deltaFrames; // if 0 frames left, stop immediately, don't wait for the next frame calculation
        // because if isAnimating become true, this will be a new animation

        if (this.framesLeftToPlay <= 0) this.data.pluginApi.stop();
      }
    }
  }

  if (this.isAnimating) requestAnimationFrame(this.boundAnimate);
}

function _calculateDuration2() {
  if (this.settings.frameTime) return this.settings.frameTime * this.settings.frames; // frameTime
  else if (this.settings.duration) return this.settings.duration; // duration
  else return this.settings.frames / this.settings.fps * 1000; // fps
}

/**
 * Animate Sprite {@link https://github.com/its2easy/animate-sprite/}
 * @example
 * let sprite = new AnimateSprite( document.getElementById('sprite'), {
 *     width: 720,
 *     height: 405,
 *     frames: 20,
 *     cols: 5,
 *     fps: 60,
 * });
 */

var _settings = /*#__PURE__*/new WeakMap();

var _data = /*#__PURE__*/new WeakMap();

var _boundCalculateSizes = /*#__PURE__*/new WeakMap();

var _dragInput = /*#__PURE__*/new WeakMap();

var _animation = /*#__PURE__*/new WeakMap();

var _initPlugin = /*#__PURE__*/new WeakSet();

var _animateSprite = /*#__PURE__*/new WeakSet();

var _changeFrame = /*#__PURE__*/new WeakSet();

var _calculatePosition = /*#__PURE__*/new WeakSet();

var _calculateSizes = /*#__PURE__*/new WeakSet();

var _toggleDrag = /*#__PURE__*/new WeakSet();

var _toggleResizeHandler = /*#__PURE__*/new WeakSet();

class AnimateSprite {
  // Classes

  /**
   * Creates plugin instance
   * @param {Element|HTMLElement} node - HTML element
   * @param {PluginOptions} options
   */
  constructor(node, options) {
    _classPrivateMethodInitSpec(this, _toggleResizeHandler);

    _classPrivateMethodInitSpec(this, _toggleDrag);

    _classPrivateMethodInitSpec(this, _calculateSizes);

    _classPrivateMethodInitSpec(this, _calculatePosition);

    _classPrivateMethodInitSpec(this, _changeFrame);

    _classPrivateMethodInitSpec(this, _animateSprite);

    _classPrivateMethodInitSpec(this, _initPlugin);

    _classPrivateFieldInitSpec(this, _settings, {
      writable: true,
      value: void 0
    });

    _classPrivateFieldInitSpec(this, _data, {
      writable: true,
      value: {
        currentFrame: 1,
        nodeWidth: null,
        nodeHeight: null,
        element: null,
        pluginApi: {}
      }
    });

    _classPrivateFieldInitSpec(this, _boundCalculateSizes, {
      writable: true,
      value: void 0
    });

    _classPrivateFieldInitSpec(this, _dragInput, {
      writable: true,
      value: void 0
    });

    _classPrivateFieldInitSpec(this, _animation, {
      writable: true,
      value: void 0
    });

    validateParameters(node);

    _classPrivateFieldSet(this, _settings, getSettings(node, options));

    validateSettings(_classPrivateFieldGet(this, _settings));
    _classPrivateFieldGet(this, _data).element = node;
    _classPrivateFieldGet(this, _data).pluginApi = this;

    _classPrivateFieldSet(this, _boundCalculateSizes, _classPrivateMethodGet(this, _calculateSizes, _calculateSizes2).bind(this));

    _classPrivateMethodGet(this, _initPlugin, _initPlugin2).call(this);
  }

  // Public API

  /**
   * Start animation
   * @returns {AnimateSprite} - plugin instance
   */
  play() {
    if (!_classPrivateFieldGet(this, _animation).isAnimating) _classPrivateFieldGet(this, _animation).play();
    return this;
  }
  /**
   * Stop animation
   * @returns {AnimateSprite} - plugin instance
   */


  stop() {
    _classPrivateFieldGet(this, _animation).stop();

    return this;
  }
  /**
   * Toggle between start and stop
   * @returns {AnimateSprite} - plugin instance
   */


  toggle() {
    if (!_classPrivateFieldGet(this, _animation).isAnimating) this.play();else this.stop();
    return this;
  }
  /**
   * Show the next frame
   * @returns {AnimateSprite} - plugin instance
   */


  next() {
    this.stop();

    _classPrivateMethodGet(this, _changeFrame, _changeFrame2).call(this, _classPrivateFieldGet(this, _animation).getNextFrame(1));

    return this;
  }
  /**
   * Show the previous frame
   * @returns {AnimateSprite} - plugin instance
   */


  prev() {
    this.stop();

    _classPrivateMethodGet(this, _changeFrame, _changeFrame2).call(this, _classPrivateFieldGet(this, _animation).getNextFrame(1, !_classPrivateFieldGet(this, _settings).reverse));

    return this;
  }
  /**
   * Set frame (without animation)
   * @param {number} frameNumber - Number of the frame to show
   * @returns {AnimateSprite} - plugin instance
   */


  setFrame(frameNumber) {
    this.stop();

    _classPrivateMethodGet(this, _changeFrame, _changeFrame2).call(this, normalizeFrameNumber(frameNumber, _classPrivateFieldGet(this, _settings).frames));

    return this;
  }
  /**
   * Starts the animation, that plays until the specified frame number
   * @param {number} frameNumber - Target frame number
   * @param {Object} [options] - Options
   * @param {boolean} [options.shortestPath=false] - If set to true and loop enabled, will use the shortest path
   * @returns {AnimateSprite} - plugin instance
   */


  playTo(frameNumber, options) {
    frameNumber = normalizeFrameNumber(frameNumber, _classPrivateFieldGet(this, _settings).frames);
    const innerPathDistance = Math.abs(frameNumber - _classPrivateFieldGet(this, _data).currentFrame),
          // not crossing edge frames
    outerPathDistance = _classPrivateFieldGet(this, _settings).frames - innerPathDistance,
          // crossing edges frames
    shouldUseOuterPath = _classPrivateFieldGet(this, _settings).loop && (options === null || options === void 0 ? void 0 : options.shortestPath) && outerPathDistance < innerPathDistance;

    if (!shouldUseOuterPath) {
      // Inner path (default)
      // long conditions to make them more readable
      if (frameNumber > _classPrivateFieldGet(this, _data).currentFrame) this.setReverse(false); // move forward
      else this.setReverse(true); // move backward
    } else {
      // Outer path
      if (frameNumber < _classPrivateFieldGet(this, _data).currentFrame) this.setReverse(false); // move forward
      else this.setReverse(true); // move backward
    }

    return this.playFrames(shouldUseOuterPath ? outerPathDistance : innerPathDistance);
  }
  /**
   * Starts the animation in the current direction with the specified number of frames in queue
   * @param {number} [numberOfFrames=0] - Number of frames to play
   * @returns {AnimateSprite} - plugin instance
   */


  playFrames() {
    let numberOfFrames = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 0;
    numberOfFrames = Math.floor(numberOfFrames);
    if (numberOfFrames < 0) return this.stop(); //empty animation

    _classPrivateFieldGet(this, _animation).framesLeftToPlay = numberOfFrames;
    return this.play();
  }
  /**
   * Change the direction of the animation. Alias to <b>setOption('reverse', true)</b>
   * @param {boolean} [reverse=true] True for backward animation, false for forward
   * @returns {AnimateSprite} - plugin instance
   */


  setReverse() {
    let reverse = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : true;
    _classPrivateFieldGet(this, _settings).reverse = !!reverse;
    return this;
  }
  /** Get current reverse option. Alias to <b>getOption('reverse')</b>
   * @returns {boolean} - reverse or not
   */


  getReverse() {
    return _classPrivateFieldGet(this, _settings).reverse;
  }
  /**
   * Calculate new dimensions (sprite element and frame), this function should be called if element size was changed by a script.
   * Called automatically after page resize
   * @returns {AnimateSprite} - plugin instance
   */


  updateSizes() {
    _classPrivateMethodGet(this, _calculateSizes, _calculateSizes2).call(this);

    return this;
  }
  /**
   * Returns option value
   * @param {string} option - Option name. All options are allowed.
   * @returns {*} - Option value
   */


  getOption(option) {
    const allowedOptions = getSettingsKeys();

    if (allowedOptions.includes(option)) {
      return _classPrivateFieldGet(this, _settings)[option];
    } else {
      console.warn(`${option} is not allowed in getOption`);
    }
  }
  /**
   * Set new option value
   * @param {string} option - Option name. All options are allowed.
   * @param {*} value - new value
   * @returns {AnimateSprite} - plugin instance
   */


  setOption(option, value) {
    const allowedOptions = getSettingsKeys();

    if (allowedOptions.includes(option)) {
      _classPrivateFieldGet(this, _settings)[option] = value;

      if (option === "width" || option === "height" || option === "frames" || option === "cols") {
        _classPrivateFieldGet(this, _settings)[option] = parseInt(value, 10); // additional validation

        _classPrivateMethodGet(this, _calculateSizes, _calculateSizes2).call(this);
      } else if (option === "frameTime" || option === "duration" || option === "fps") {
        _classPrivateFieldGet(this, _settings).frameTime = _classPrivateFieldGet(this, _settings).duration = _classPrivateFieldGet(this, _settings).fps = false; // Reset

        _classPrivateFieldGet(this, _settings)[option] = parseInt(value, 10); // set again because of reset

        _classPrivateFieldGet(this, _animation).updateDuration();
      } else if (option === "draggable") _classPrivateMethodGet(this, _toggleDrag, _toggleDrag2).call(this, value);else if (option === 'dragModifier') _classPrivateFieldGet(this, _settings).dragModifier = Math.abs(+value);
    } else {
      console.warn(`${option} is not allowed in setOption`);
    }

    return this;
  }
  /** @returns {number} - current frame number */


  getCurrentFrame() {
    return _classPrivateFieldGet(this, _data).currentFrame;
  }
  /** @returns {boolean} - animating or not */


  isAnimating() {
    return _classPrivateFieldGet(this, _animation).isAnimating;
  }
  /**
   * Stop the animation and return to the first frame
   * @returns @returns {AnimateSprite} - plugin instance
   */


  reset() {
    this.stop();

    _classPrivateMethodGet(this, _changeFrame, _changeFrame2).call(this, normalizeFrameNumber(1, _classPrivateFieldGet(this, _settings).frames));

    return this;
  }
  /**
   * Stop animation, remove event listeners. Method doesn't remove sprite element from the DOM
   */


  destroy() {
    this.stop();

    _classPrivateMethodGet(this, _toggleDrag, _toggleDrag2).call(this, false);

    _classPrivateMethodGet(this, _toggleResizeHandler, _toggleResizeHandler2).call(this, false);
  }

} // todo can't import typedef from another file because it won't add type to d.ts, check this in the future

/**
 * @typedef {object} PluginOptions
 * @property {number} width - Width of one frame in sprite
 * @property {number} height - Height of one frame in sprite
 * @property {number} frames - Number of frames
 * @property {number|false} [cols=false] - Number of cols if more than 1 row
 * @property {boolean} [loop=false] - Whether to loop the animation
 * @property {boolean} [autoplay=false] - Autoplay
 * @property {number|false} [frameTime] - ms, time between frames
 * @property {number|false} [duration] - ms, total time, alternative to frameTime
 * @property {number|false} [fps=24] - fps, alternative to frameTime
 * @property {boolean} [reverse=false] - Reverse direction of animation
 * @property {number|boolean} [draggable=false] - Draggable by mouse or touch
 * @property {boolean} [inversion=false] - Inversion changes drag direction
 * @property {number} [dragModifier=1] - Sensitivity factor for user interaction. Only positive numbers are allowed
 * @property {'pageScrollTimer' | 'preventPageScroll' | 'allowPageScroll'} [touchScrollMode="pageScrollTimer"]
 * - Page scroll behavior with touch events (preventPageScroll,allowPageScroll, pageScrollTimer)
 * @property {number} [pageScrollTimerDelay=1500] - Time in ms when touch scroll will be disabled during interaction
 * if options.touchScrollMode = "pageScrollTimer"
 * @property {function(AnimateSprite):void} [onAnimationEnd] - Occurs when animation has ended
 * @property {function(AnimateSprite):void} [onAfterFrameChange] - Occurs after the frame has switched
 */

function _initPlugin2() {
  _classPrivateFieldSet(this, _animation, new Animation({
    settings: _classPrivateFieldGet(this, _settings),
    data: _classPrivateFieldGet(this, _data),
    changeFrame: _classPrivateMethodGet(this, _changeFrame, _changeFrame2).bind(this)
  }));

  _classPrivateMethodGet(this, _calculateSizes, _calculateSizes2).call(this);

  _classPrivateMethodGet(this, _toggleResizeHandler, _toggleResizeHandler2).call(this, true);

  if (_classPrivateFieldGet(this, _settings).autoplay) this.play();
  if (_classPrivateFieldGet(this, _settings).draggable) _classPrivateMethodGet(this, _toggleDrag, _toggleDrag2).call(this, true);
}

function _animateSprite2(frame) {
  _classPrivateFieldGet(this, _data).element.style.backgroundPosition = _classPrivateMethodGet(this, _calculatePosition, _calculatePosition2).call(this, frame);
  if (_classPrivateFieldGet(this, _settings).onAfterFrameChange) _classPrivateFieldGet(this, _settings).onAfterFrameChange(_classPrivateFieldGet(this, _data).pluginApi);
}

function _changeFrame2(frameNumber) {
  if (frameNumber === _classPrivateFieldGet(this, _data).currentFrame) return; //skip same frame

  _classPrivateMethodGet(this, _animateSprite, _animateSprite2).call(this, frameNumber);

  _classPrivateFieldGet(this, _data).currentFrame = frameNumber;
}

function _calculatePosition2(frame) {
  let xPadding,
      yPadding = 0;

  if (!_classPrivateFieldGet(this, _settings).cols) {
    // Single row sprite
    xPadding = (frame - 1) * _classPrivateFieldGet(this, _data).nodeWidth;
  } else {
    // Multiline sprite
    xPadding = (frame - 1) % _classPrivateFieldGet(this, _settings).cols * _classPrivateFieldGet(this, _data).nodeWidth;
    yPadding = Math.floor((frame - 1) / _classPrivateFieldGet(this, _settings).cols) * _classPrivateFieldGet(this, _data).nodeHeight;
  }

  return `-${xPadding}px -${yPadding}px`;
}

function _calculateSizes2() {
  const newNodeWidth = _classPrivateFieldGet(this, _data).element.offsetWidth;

  if (_classPrivateFieldGet(this, _data).nodeWidth === newNodeWidth) return; //skip if same size

  const wasAnimating = _classPrivateFieldGet(this, _animation).isAnimating,
        widthHeightRatio = _classPrivateFieldGet(this, _settings).width / _classPrivateFieldGet(this, _settings).height;

  _classPrivateFieldGet(this, _data).nodeWidth = newNodeWidth;
  _classPrivateFieldGet(this, _data).nodeHeight = _classPrivateFieldGet(this, _data).nodeWidth / widthHeightRatio;
  _classPrivateFieldGet(this, _data).element.style.height = _classPrivateFieldGet(this, _data).nodeHeight + "px";
  let bgWidth = !_classPrivateFieldGet(this, _settings).cols ? _classPrivateFieldGet(this, _settings).frames * _classPrivateFieldGet(this, _data).nodeWidth : _classPrivateFieldGet(this, _settings).cols * _classPrivateFieldGet(this, _data).nodeWidth;
  let bgHeight = !_classPrivateFieldGet(this, _settings).cols ? _classPrivateFieldGet(this, _data).nodeHeight : Math.ceil(_classPrivateFieldGet(this, _settings).frames / _classPrivateFieldGet(this, _settings).cols) * _classPrivateFieldGet(this, _data).nodeHeight;
  _classPrivateFieldGet(this, _data).element.style.backgroundSize = `${bgWidth}px ${bgHeight}px`;
  if (wasAnimating) this.play();else _classPrivateMethodGet(this, _animateSprite, _animateSprite2).call(this, _classPrivateFieldGet(this, _data).currentFrame);
  if (_classPrivateFieldGet(this, _dragInput)) _classPrivateFieldGet(this, _dragInput).updateThreshold();
}

function _toggleDrag2() {
  let enable = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : true;

  if (enable) {
    if (!_classPrivateFieldGet(this, _dragInput)) _classPrivateFieldSet(this, _dragInput, new DragInput({
      data: _classPrivateFieldGet(this, _data),
      settings: _classPrivateFieldGet(this, _settings),
      changeFrame: _classPrivateMethodGet(this, _changeFrame, _changeFrame2).bind(this),
      getNextFrame: _classPrivateFieldGet(this, _animation).getNextFrame.bind(_classPrivateFieldGet(this, _animation))
    }));

    _classPrivateFieldGet(this, _dragInput).enableDrag();
  } else {
    if (_classPrivateFieldGet(this, _dragInput)) _classPrivateFieldGet(this, _dragInput).disableDrag();
  }
}

function _toggleResizeHandler2() {
  let add = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : true;
  if (add) window.addEventListener("resize", _classPrivateFieldGet(this, _boundCalculateSizes));else window.removeEventListener("resize", _classPrivateFieldGet(this, _boundCalculateSizes));
}

export { AnimateSprite as default };
//# sourceMappingURL=animate-sprite.esm.js.map
